%---------------------------------------------------%
% Foot Motion Problem:                              %
%---------------------------------------------------%

clear all; close all;

handLoadControls = true;
grResidualControls = false;
springPosParams = false;

% Load model parameters determined by past calibration runs
cdorig = cd;
cd('../SetupFiles')
auxdata = SetupOpenSimOptimalTracking(0,true);
cd(cdorig)

auxdata.handLoadControls = handLoadControls;
auxdata.grResidualControls = grResidualControls;
auxdata.springPosParams = springPosParams;

% Load initial and final model pose and time
t0 = auxdata.time(1);
tf = auxdata.time(end);
time = auxdata.time;
q0 = auxdata.q(1,:);
qp0 = auxdata.qp(1,:);
qf = auxdata.q(end,:);
qpf = auxdata.qp(end,:);

% Set initial guesses for Loads (control) and joint angle trajectories
% (states)
loadsGuess = auxdata.idData;
qGuess = auxdata.q;
qpGuess = auxdata.qp;
qppGuess = auxdata.qpp;
qpppGuess = auxdata.qppp;
nJerkControls = size(qpppGuess,2);
auxdata.nJerkControls = nJerkControls;

% Determine the range of the states
qRange = max(qGuess) - min(qGuess);
qpRange = max(qpGuess) - min(qpGuess);
qppRange = max(qppGuess) - min(qppGuess);
qpppRange = max(qpppGuess) - min(qpppGuess);

auxdata.activeDofs = logical(1-(qRange == 0));
auxdata.activeDofs(1:6) = [];

% Give locked DOFs a range of motion
qpppRange(qRange == 0) = 40000*pi/180;
qppRange(qRange == 0) = 4000*pi/180;
qpRange(qRange == 0) = 400*pi/180;
qRange(qRange == 0) = 60*pi/180;

% Set bounds on the states based on the previously calcualted range
qMin = min(qGuess) - 2*qRange;  
qMax  = max(qGuess) + 2*qRange;
qpMin = min(qpGuess) - 1.5*qpRange; 
qpMax  = max(qpGuess) + 1.5*qpRange;
qppMin = min(qppGuess) - 1*qppRange; 
qppMax  = max(qppGuess) + 1*qppRange;
qpppMin = min(qpppGuess) - 1*qpppRange; % 0.5 %NOTE (Nathan): This is different than previous step (ground contact calibration)
qpppMax  = max(qpppGuess) + 1*qpppRange; %0.5

% Determing the range of the controls
loadsRange = max(auxdata.idData) - min(auxdata.idData);
torquesRange = max(auxdata.torques) - min(auxdata.torques);

% Give bigger range to DOFs with small loads (for instance the toes)
loadsRange(loadsRange<1/(auxdata.tScale^2)) = 25/(auxdata.tScale^2);
torquesRange(torquesRange<1/(auxdata.tScale^2)) = 25/(auxdata.tScale^2);

% Set bounds on the torques (for when using extra controls)
loadsMin = min(auxdata.idData)-2*loadsRange; 
loadsMax = max(auxdata.idData)+2*loadsRange;

torquesMin = min(auxdata.torques) - 2*torquesRange;
torquesMax = max(auxdata.torques) + 2*torquesRange;

torquesGuess = auxdata.torques;

% Set weights for marker errors (the model tracks markers from experimental
% data
% MkrNamesOSIM = {'R.Shoulder' 'L.Shoulder' 'Neck' 'Chest' 'R.Elbow' 'R.Wrist' 'L.Elbow' 'L.Wrist' 'R.ASIS' 'L.ASIS'...
%     'Sacral' 'R.Thigh.Superior' 'R.Thigh.Inferior' 'R.Thigh.Lateral' 'R.Shank.Superior' 'R.Shank.Inferior' 'R.Shank.Lateral'...
%     'R.Heel' 'R.Midfoot.Superior' 'R.Midfoot.Lateral' 'R.Toe' 'L.Thigh.Superior' 'L.Thigh.Inferior' 'L.Thigh.Lateral'...
%     'L.Shank.Superior' 'L.Shank.Lateral' 'L.Shank.Inferior' 'L.Heel' 'L.Midfoot.Superior' 'L.Midfoot.Lateral' 'L.Toe'...
%     'L.PSIS' 'R.PSIS'};
auxdata.errorWeights = ones(1,numel(auxdata.mkrNames));
auxdata.errorWeights([18 21 28 31]) = 10;
auxdata.errorWeights([3:8]) = .1;

auxdata.errorWeights = repmat(auxdata.errorWeights,3,1);
auxdata.errorWeights = auxdata.errorWeights(:)';

% Initilize osim models in mex files
opensimPointKin_v3('Subject1_v4pt1_GPOPS.osim');
opensimID_OMP('Subject1_v4pt1_GPOPS.osim');
opensimGetBodyPosVel_OMP('Subject1_v4pt1_GPOPS.osim');

% Get median hand positions
handForceLoc = [0.01,-0.07,0; 0.01,-0.07,0]; % NOTE (Nathan): Make sure that this is the same location as where the force is applied in the model
auxdata.handForceLoc = handForceLoc;
handBodyMat = auxdata.handBodyMat;
% [handPos, handVel, handAng, handAngVel] = opensimGetBodyPosVel_OMP(time,[qGuess qpGuess],handBodyMat);
[handPos, handVel, handAng, handAngVel] = opensimGetBodyPosVel_OMP(time,[qGuess qpGuess],handForceLoc,handBodyMat);

medianHandPos = median(handPos);
medianHandAng = median(handAng);
auxdata.medianHandPos_r = medianHandPos(1:3);
auxdata.medianHandPos_l = medianHandPos(4:6);
auxdata.medianHandAng_r = medianHandAng(1:3);
auxdata.medianHandAng_l = medianHandAng(4:6);

% Min and Max hand loads allowed
% NOTE (Nathan): Thses are only used if the hand loads are minimized or for
% hand load controls
handLoadMin = -[200*ones(1,3),50*ones(1,3),200*ones(1,3),50*ones(1,3)];
handLoadMax = [200*ones(1,3),50*ones(1,3),200*ones(1,3),50*ones(1,3)];
handLoadRange = handLoadMax - handLoadMin;

% Residual ground reaction actuator bounds
grResidualMin = -[10,10,10,1,1,1,10,10,10,1,1,1];
grResidualMax = [10,10,10,1,1,1,10,10,10,1,1,1];
grResidualRange = grResidualMax - grResidualMin;


% Marker Errors
mkrErrorsAllowed = 2e-2*ones(1,numel(auxdata.mkrNames));
mkrErrorsAllowed([18:21 28:31]) = 1.5e-2;
mkrErrorsAllowed([3:8]) = 3e-2;
mkrErrorsAllowed = repmat(mkrErrorsAllowed, 3,1);% Duplicate for each dimension
mkrErrorsAllowed = mkrErrorsAllowed(:)';

auxdata.numSprings = sum(auxdata.numSpringsBody);

% load previous solution for use as an initial guess
% load solution_osimOptTracking_calibrateSprings29.mat
auxdata.beltSpeed = 0.6;

% solution = output.result.interpsolution;

% Setup scale factors
auxdata.tMin = t0;
auxdata.tMax = tf;
auxdata.stateMin = [qMin,qpMin,qppMin];
auxdata.stateMax = [qMax,qpMax,qppMax];
if handLoadControls
    if grResidualControls
        auxdata.controlMin = [qpppMin,handLoadMin,grResidualMin];
        auxdata.controlMax = [qpppMax,handLoadMax,grResidualMax];
    %     auxdata.integralMin = 0*ones(1,99+6+6+12+31); %99+6+6+23+31
        auxdata.integralMin = 0*ones(1,99+6+6+12+2+12+12+31);
        auxdata.integralMax = auxdata.tMax*([mkrErrorsAllowed ...
            1e0*[1e1 2e1 1e1]...
            1e0*[1e1 2e1 1e1]... 
            1e1*ones(1,3)...
            1e1*ones(1,3)...
            1*loadsRange([7:10,12:13,15:18,20:21])... % [7:10,12:13,15:18,20:21,23:29,33:36] Ignore pelvis, toes, wrist, patella loads. OLD: [1:6,8:13,15:end]
            1*qRange([13,20]) ... % Toe angle tracking
            handLoadRange...
            grResidualRange...
            1*qpppRange]).^2;
    else
        auxdata.controlMin = [qpppMin,handLoadMin];
        auxdata.controlMax = [qpppMax,handLoadMax];
    %     auxdata.integralMin = 0*ones(1,99+6+6+12+31); %99+6+6+23+31
        auxdata.integralMin = 0*ones(1,99+6+6+12+2+12+31);
        auxdata.integralMax = auxdata.tMax*([mkrErrorsAllowed ...
            1e0*[1e1 2e1 1e1]...
            1e0*[1e1 2e1 1e1]... 
            1e1*ones(1,3)...
            1e1*ones(1,3)...
            1*loadsRange([7:10,12:13,15:18,20:21])... % [7:10,12:13,15:18,20:21,23:29,33:36] Ignore pelvis, toes, wrist, patella loads. OLD: [1:6,8:13,15:end]
            1*qRange([13,20]) ... % Toe angle tracking
            handLoadRange...
            1*qpppRange]).^2; % Ignore locked coordinates (wrists) and patella OLD: all
    end
    
    
else
    if grResidualControls
        auxdata.controlMin = [qpppMin,grResidualMin];
        auxdata.controlMax = [qpppMax,grResidualMax];
    %     auxdata.integralMin = 0*ones(1,99+6+6+12+31); %99+6+6+23+31
        auxdata.integralMin = 0*ones(1,99+6+6+12+2+12+12+31);
        auxdata.integralMax = auxdata.tMax*([mkrErrorsAllowed ...
            1e0*[1e1 2e1 1e1]...
            1e0*[1e1 2e1 1e1]... 
            1e1*ones(1,3)...
            1e1*ones(1,3)...
            1*loadsRange([7:10,12:13,15:18,20:21])... % [7:10,12:13,15:18,20:21,23:29,33:36] Ignore pelvis, toes, wrist, patella loads. OLD: [1:6,8:13,15:end]
            1*qRange([13,20]) ... % Toe angle tracking
            handLoadRange...
            grResidualRange...
            1*qpppRange]).^2;
    else
        auxdata.controlMin = qpppMin;
        auxdata.controlMax = qpppMax;
        auxdata.integralMin = 0*ones(1,99+6+6+12+2+12+31); %99+6+6+23+31
        % auxdata.integralMin = 0*ones(1,99+6+6+12+12+31);
        auxdata.integralMax = auxdata.tMax*([mkrErrorsAllowed ...
            1e0*[1e1 2e1 1e1]...
            1e0*[1e1 2e1 1e1]... 
            1e1*ones(1,3)...
            1e1*ones(1,3)...
            1*loadsRange([7:10,12:13,15:18,20:21])... % [7:10,12:13,15:18,20:21,23:29,33:36] Ignore pelvis, toes, wrist, patella loads. OLD: [1:6,8:13,15:end]
            1*qRange([13,20]) ... % Toe angle tracking
            handLoadRange...
            1*qpppRange]).^2; % Ignore locked coordinates (wrists) and patella OLD: all
    end
    
end

onescol = ones(length(auxdata.time),1);
numTorques = size(auxdata.torques,2);
pathMult = 1e0;
% auxdata.pathMin = pathMult*[-1e-1*ones(1,3) -1e0*ones(1,3)]; % 0.11*
% auxdata.pathMax = pathMult*[1e-1*ones(1,3) 1e0*ones(1,3)];
auxdata.pathMin = pathMult*[-1e-1*ones(1,3) -1e0*ones(1,3),-10*pi/180*ones(1,2)]; % 0.11*
auxdata.pathMax = pathMult*[1e-1*ones(1,3) 1e0*ones(1,3),10*pi/180*ones(1,2)];


% Load Previous Solution
load('solution_dynamicallyConsistentIKID_snopt_v4pt20.mat');

if 0 %Run Opt    
%-------------------------------------------------------------------------%
%----------------------- Setup for Problem Bounds ------------------------%
%-------------------------------------------------------------------------%

nStates = length(auxdata.stateMin);
nControls = length(auxdata.controlMin);
% nparams = length(auxdata.paramsmin);

iphase = 1;
bounds.phase(iphase).initialtime.lower = -.5;
bounds.phase(iphase).initialtime.upper = -.5;
bounds.phase(iphase).finaltime.lower = .5;
bounds.phase(iphase).finaltime.upper = .5;
% states are the joint angles, velocities, accelerations and the joint
% torques
bounds.phase(iphase).initialstate.lower = -.5*ones(1,nStates);%,vmin];
bounds.phase(iphase).initialstate.upper = .5*ones(1,nStates);%,vmax];
bounds.phase(iphase).finalstate.lower = -.5*ones(1,nStates);%,vmin];%[qf,qpf];
bounds.phase(iphase).finalstate.upper = .5*ones(1,nStates);%,vmax];%[qf,qpf];
bounds.phase(iphase).state.lower = -.5*ones(1,nStates);%,vmin];
bounds.phase(iphase).state.upper = .5*ones(1,nStates);%,vmax];
% Dynamics equations are forced to be satisfied by path constraints
% Control with joint angle jerk and the time derivative of the joint loads
bounds.phase(iphase).control.lower = -.5*ones(1,nControls);
bounds.phase(iphase).control.upper = .5*ones(1,nControls);
bounds.phase(iphase).path.lower = -0.5*ones(size(auxdata.pathMin)); % Bounds on pelvis residuals
bounds.phase(iphase).path.upper = 0.5*ones(size(auxdata.pathMax));
bounds.phase(iphase).integral.lower = 0*(ones(size(auxdata.integralMin)));
bounds.phase(iphase).integral.upper = 1*(ones(size(auxdata.integralMax)));


%-------------------------------------------------------------------------%
%---------------------- Provide Guess of Solution ------------------------%
%-------------------------------------------------------------------------%
% solution.phase.control(:,30:99) = solution.phase.control(:,30:99)/10;
% Set initial guess either from previous optimal control solution or from
% the auxilary data which was calculated from a different model
iphase = 1;
if 1
    guess = solution;
else
    guess.phase(iphase).time    = ([auxdata.time]-(auxdata.tMax+auxdata.tMin)/2)./(auxdata.tMax-auxdata.tMin);
    guess.phase(iphase).state   = ([qGuess qpGuess qppGuess]-onescol*(auxdata.stateMax+auxdata.stateMin)/2)./(onescol*(auxdata.stateMax-auxdata.stateMin));% auxdata.Loads];% auxdata.Loadsp]
    if handLoadControls
        if grResidualControls
            guess.phase(iphase).control = ([qpppGuess,onescol*zeros(1,12),onescol*zeros(1,12)]-onescol*(auxdata.controlMax+auxdata.controlMin)/2)./(onescol*(auxdata.controlMax-auxdata.controlMin)); %zeros(size(auxdata.LoadsExp(:,:)));
        else
            guess.phase(iphase).control = ([qpppGuess,onescol*zeros(1,12)]-onescol*(auxdata.controlMax+auxdata.controlMin)/2)./(onescol*(auxdata.controlMax-auxdata.controlMin)); %zeros(size(auxdata.LoadsExp(:,:)));
        end
        
    else
        if grResidualControls
            guess.phase(iphase).control = ([qpppGuess,onescol*zeros(1,12)]-onescol*(auxdata.controlMax+auxdata.controlMin)/2)./(onescol*(auxdata.controlMax-auxdata.controlMin)); %zeros(size(auxdata.LoadsExp(:,:)));
        else
            guess.phase(iphase).control = ([qpppGuess]-onescol*(auxdata.controlMax+auxdata.controlMin)/2)./(onescol*(auxdata.controlMax-auxdata.controlMin)); %zeros(size(auxdata.LoadsExp(:,:)));
        end
        
    end
    
    guess.phase(iphase).integral = (10 - (auxdata.integralMax+auxdata.integralMin)/2)./(auxdata.integralMax - auxdata.integralMin);
    
end


%-------------------------------------------------------------------------%
%------------- Set up Event Constraints That Link Phases -----------------%
%-------------------------------------------------------------------------%
% bounds.eventgroup(1).lower = [zeros(1,25)-0.5*pi/180 zeros(1,25)-1*pi/180 -2*[10 25 10 10 25 10] pathMult*[-1e-1*ones(1,3) -1e0*ones(1,3)]];
% bounds.eventgroup(1).upper = [zeros(1,25)+0.5*pi/180 zeros(1,25)+1*pi/180 2*[10 25 10 10 25 10] pathMult*[1e-1*ones(1,3) 1e0*ones(1,3)]];
bounds.eventgroup(1).lower = [zeros(1,25)-0.5*pi/180 zeros(1,25)-1*pi/180 -1*[10 25 10 10 25 10]];
bounds.eventgroup(1).upper = [zeros(1,25)+0.5*pi/180 zeros(1,25)+1*pi/180 1*[10 25 10 10 25 10]];


%-------------------------------------------------------------------------%
%------------- Assemble Information into Problem Structure ---------------%
%-------------------------------------------------------------------------%
setup.name = 'openSim Full Body Motion';
setup.functions.continuous = @osimOptimalTrackingContinuous;
auxdata.ContinuousFunc = setup.functions.continuous;
setup.functions.endpoint = @osimOptimalTrackingEndpoint;
setup.auxdata = auxdata;
setup.bounds = bounds;
setup.guess = guess;
setup.nlp.solver = 'snopt';
% setup.nlp.ipoptoptions.linear_solver = 'ma57';
setup.nlp.ipoptoptions.tolerance = 1e-2;
setup.nlp.snoptoptions.tolerance = 5e-5;
setup.nlp.ipoptoptions.maxiterations = 300;
% setup.derivatives.stepsize1 = 1e-9;
setup.derivatives.supplier = 'sparseFD';
setup.derivatives.derivativelevel = 'first';
setup.derivatives.dependencies = 'sparse';
% setup.scales.method='automatic-bounds';
mesh.method = 'hp-PattersonRao';
mesh.tolerance       = 1e-4;
mesh.maxiterations    = 0;
mesh.colpointsmin    = 2;
mesh.colpointsmax    = 10;
N                    = 10; %10
mesh.phase.colpoints = 5*ones(1,N); %5
mesh.phase.fraction  = ones(1,N)/N;
setup.method = 'RPM-integration';
setup.mesh = mesh;


%-------------------------------------------------------------------------%
%------------------------- Solve Problem Using GPOP2 ---------------------%
%-------------------------------------------------------------------------%
tic
output = gpops2(setup);
toc
solution = output.result.solution;
save solution_dynamicallyConsistentIKID_snopt_v4pt20.mat solution
save output_dynamicallyConsistentIKID_snopt_v4pt20.mat output

end

%-------------------------------------------------------------------------%
%------------------------------- Plot Solution ---------------------------%
%-------------------------------------------------------------------------%

%load output_osimOptTracking_calibrateSprings4.mat

tScale = 1;%auxdata.tScale;

input = solution;
input.auxdata = auxdata;

% outputContinuous = FullBodyOptimalControlContinuous_experimentalGRFs_Markers(input);
outputContinuous = osimOptimalTrackingContinuous(input);

onescol = ones(size(solution.phase.state(:,1)));

solution.phase.time = (solution.phase.time)*(auxdata.tMax-auxdata.tMin)+(auxdata.tMax+auxdata.tMin)/2;
solution.phase.state = (solution.phase.state).*(onescol*(auxdata.stateMax-auxdata.stateMin))+onescol*(auxdata.stateMax+auxdata.stateMin)/2;
solution.phase.control = (solution.phase.control).*(onescol*(auxdata.controlMax-auxdata.controlMin))+onescol*(auxdata.controlMax+auxdata.controlMin)/2;

mkrDataSplined = interp1(auxdata.time,auxdata.mkrData,solution.phase.time,'pchip');

mkrDataSplined = fnval(auxdata.spMkrData,solution.phase.time)';

error = mkrDataSplined-outputContinuous.markers;

% Calculate RMS error for each marker at each time frame
for i = 1:size(mkrDataSplined,2)/3
    RMSError(:,i) = sum(error(:,(i-1)*3+1:3*i).^2,2).^(1/2);
end

GRFsSplit = outputContinuous.GRFsSplit;
figure % 1
plot(solution.phase.time,GRFsSplit(:,1:3)+GRFsSplit(:,4:6),'r')
hold on
plot(auxdata.time,auxdata.grfDataR(:,1:3),'b')
title('Right GRFs');

figure % 2
plot(solution.phase.time,GRFsSplit(:,13:15)+GRFsSplit(:,16:18),'r')
hold on
plot(auxdata.time,auxdata.grfDataL(:,1:3),'b')
title('Left GRFs');

figure % 3
plot(solution.phase.time,GRFsSplit(:,7:9)+GRFsSplit(:,10:12),'r')
hold on
plot(auxdata.time,auxdata.grfDataR(:,4:6),'b')
title('Right GRMs');

figure % 4
plot(solution.phase.time,GRFsSplit(:,19:21)+GRFsSplit(:,22:24),'r')
hold on
plot(auxdata.time,auxdata.grfDataL(:,4:6),'b')
title('Left GRMs');

figure % 5
for i = 1:29
    if ismember(i,4:6)
        midpoint = ((max(qGuess(:,i)) - min(qGuess(:,i)))/2 + min(qGuess(:,i)));
        minpoint = midpoint - 0.1;
        maxpoint = midpoint + 0.1;
        subplot(5,6,i)
        plot(solution.phase.time/solution.phase.time(end), solution.phase.state(:,i),'r')
        hold on
    %     plot(auxdata.time/auxdata.time(end), auxdata.q(:,i),'b')
        plot(auxdata.time/auxdata.time(end), qGuess(:,i),'b')
        ylim([minpoint,maxpoint]);
        title(strrep(auxdata.qLabels(i),'_',' '));
    else
        midpoint = (180/pi)*((max(qGuess(:,i)) - min(qGuess(:,i)))/2 + min(qGuess(:,i)));
        minpoint = midpoint - 0.75*180/pi;
        maxpoint = midpoint + 0.75*180/pi;
        subplot(5,6,i)
        plot(solution.phase.time/solution.phase.time(end), solution.phase.state(:,i)*180/pi,'r')
        hold on
    %     plot(auxdata.time/auxdata.time(end), auxdata.q(:,i),'b')
        plot(auxdata.time/auxdata.time(end), qGuess(:,i)*180/pi,'b')
        ylim([minpoint,maxpoint]);
        title(strrep(auxdata.qLabels(i),'_',' '));
    end
end

figure % 6
for i = 1:29
    
    subplot(5,6,i)
    plot(solution.phase.time/solution.phase.time(end), solution.phase.control(:,i),'r')
    hold on
%     plot(auxdata.time/auxdata.time(end), auxdata.qpp(:,i),'b')
    plot(auxdata.time/auxdata.time(end), qpppGuess(:,i),'b')
    title(strrep(auxdata.qLabels(i),'_',' '));
end

figure % 7
for i = 1:23
    subplot(5,5,i)
    plot(solution.phase.time/solution.phase.time(end), outputContinuous.torques(:,i),'r')
    hold on
%     plot(auxdata.time/auxdata.time(end), auxdata.idData(:,i),'b')
    plot(auxdata.time/auxdata.time(end), auxdata.torques(:,i),'b')
    title(strrep(auxdata.torqueNames(i),'_',' '));
end

figure % 8
for iii = 1:3
    subplot(2,3,iii);
    plot(solution.phase.time/solution.phase.time(end), outputContinuous.handLoadsR(:,iii));
    subplot(2,3,iii+3);
    plot(solution.phase.time/solution.phase.time(end), outputContinuous.handLoadsL(:,iii));
    title('Hand Forces');
end

figure % 9
for iii = 1:3
    subplot(2,3,iii);
    plot(solution.phase.time/solution.phase.time(end), outputContinuous.handLoadsR(:,iii+3));
    subplot(2,3,iii+3);
    plot(solution.phase.time/solution.phase.time(end), outputContinuous.handLoadsL(:,iii+3));
    title('Hand Torques');
end

[spTorques] = spaps(solution.phase.time,outputContinuous.torques',0.01);
torques = fnval(spTorques,auxdata.time)';
[spq] = spaps(solution.phase.time,solution.phase.state(:,1:31)',0.0001);
q = fnval(spq,auxdata.time)';
% 
% figure
% for i = 1:25
%     subplot(5,5,i)
%     plot(auxdata.Time/auxdata.Time(end), Torques(:,i),'r')
%     hold on
%     plot(auxdata.Time/auxdata.Time(end), auxdata.IDData(:,i),'b')
% end
% 
save GPOPSResults_Trial04_Cycle12.mat torques q

[spIDnew] = spaps(solution.phase.time,outputContinuous.torques',.5);

control = fnval(spIDnew,auxdata.time)';

NewID = control(:,[8 9 11 12 13 1 2 4 5 6]);

save Control_Trial04_Cycle12.mat control

save NewIDLoads_Trial04_Cycle12.mat NewID

outputIKMot('IK_Trial04_Cycle12.mot',solution.phase.state(:,1:31),solution.phase.time)